#include <SoftwareSerial.h>


#define TX_PIN 2                                             // Pins for the serial port defined
#define RX_PIN 3
#define DRIVER 4

#define SEARCH_MODE 0x65
#define LESS_THAN   0x66
#define GET_VER     0x5D
#define DO_RANGE    0x51
#define GET_RANGE   0x69

SoftwareSerial mySerial = SoftwareSerial(RX_PIN, TX_PIN);    // Serial port setup 

byte buffer[10];

unsigned long moduleAddress = 0;

void setup() {
  pinMode(RX_PIN, INPUT);
  pinMode(TX_PIN, OUTPUT);
  pinMode(DRIVER, OUTPUT);             // Setting driver pin high means we are writing, low for reading
  mySerial.begin(38400);               // setup serial port for SRF485WPR
  Serial.begin(9600);                  // Serial port for arduino serial monitor
  
  setSearchMode();                    
  moduleAddress = findSrf485wpr();
}

void loop() {
  digitalWrite(DRIVER, HIGH);
  buffer[0] = DO_RANGE;                                                    // Command to start a ranging in CM
  buffer[1] = (byte) ((moduleAddress >>16)&0xFF);                      // 3 address bytes
  buffer[2] = (byte) ((moduleAddress >>8)&0xFF);
  buffer[3] = (byte) moduleAddress&0xFF;
  buffer[4] = 0;                                                       // Data
  buffer[5] = (byte) ~(buffer[0] + buffer[1] + buffer[2] + buffer[3] + buffer[4]);  // Checksum
    
  sendBreak();                                                        // Send a break
  mySerial.write(buffer, 6);                                          // Send the data
  delay(100);                                                         // Wait for ranging to complete
  
  buffer[0] = GET_RANGE;                                                   // Command  to read back temperature compensated range
  buffer[1] = (byte) ((moduleAddress >>16)&0xFF);                     // 3 address bytes
    buffer[2] = (byte) ((moduleAddress >>8)&0xFF);
    buffer[3] = (byte) moduleAddress&0xFF;
    buffer[4] = 0;
    buffer[5] = (byte) ~(buffer[0] + buffer[1] + buffer[2] + buffer[3] + buffer[4]);
  sendBreak();
  mySerial.write(buffer, 6);
  
  digitalWrite(DRIVER, LOW);
  
  if(mySerial.available() < 2);                                        // Wait for the range bytes to come back and read them
  int hByte = mySerial.read();
  int lByte = mySerial.read();
  
  int range = (hByte << 8) + lByte;                                    // Put the range together and display it
  Serial.print("range = ");
  Serial.print(range, DEC);
  Serial.print("\n");
  
  delay(100);
}

void setSearchMode() {
  // Set modules into search mode
  Serial.print("*******************************\n");
  Serial.print("Setting search mode.\n");
  digitalWrite(DRIVER, HIGH);
  buffer[0] = SEARCH_MODE;        // Command to set search mode
  buffer[1] = 0;
  buffer[2] = 0;
  buffer[3] = 0;
  buffer[4] = 0;
  buffer[5] = 0x9A;        // Checksum 
  sendBreak();
  mySerial.write(buffer, 6);
}
  
unsigned long findSrf485wpr(){
  unsigned long walk = 0x800000;
  unsigned long address = 0x800000;
  Serial.print("Finding address. \n");
  
  do {
    digitalWrite(DRIVER, HIGH);
    
    buffer[0] = LESS_THAN;
    buffer[1] = (byte) ((address >>16)&0xFF);                // 3 address bytes
    buffer[2] = (byte) ((address >>8)&0xFF);
    buffer[3] = (byte) address&0xFF;
    buffer[4] = 0;
    buffer[5] = (byte) ~(buffer[0] + buffer[1] + buffer[2] + buffer[3] + buffer[4]);
    sendBreak();
    mySerial.write(buffer, 6);
   
    digitalWrite(DRIVER, LOW);
    delay(20);
    if(mySerial.available()) {                               // Wait for the return byte
      mySerial.read(); 
      address ^= walk;
    }
    walk >>= 1;
    address |= walk; 
    
  }while(walk);
  
  Serial.print("Module at address : ");                        // Display module Address
  Serial.print(address, HEX);
  Serial.print("\n");
  Serial.print("*******************************\n");
  
  digitalWrite(DRIVER, HIGH);
  buffer[0] = (byte) GET_VER;                                          // Read back the module version to take this module out of search mode
  buffer[1] = (byte) ((address >>16)&0xFF);                     // 3 address bytes
  buffer[2] = (byte) ((address >>8)&0xFF);
  buffer[3] = (byte) address&0xFF;
  buffer[4] = 0;
  buffer[5] = (byte) ~(buffer[0] + buffer[1]+ buffer[2]+ buffer[3]+ buffer[4]);
  
  sendBreak();
  mySerial.write(buffer, 6);
  
  digitalWrite(DRIVER, LOW);
  while(mySerial.available() < 4);                               // Wait for data to come back and then read it
  int id = mySerial.read();
  int hardware = mySerial.read();
  int ver = mySerial.read();
  int group = mySerial.read();
  
  
  return address;
}
  
  
void sendBreak(){
  mySerial.begin(14400);  // simulate sending a break by sending a 0 at a slower baud rate
  mySerial.write((byte)0);
  mySerial.begin(38400);
}
